;Copyright (c) 2016 Julius A. Kosan
;
;Permission is hereby granted, free of charge, to any person obtaining
;a copy of this software and associated documentation files (the
;"Software"), to deal in the Software without restriction, including
;without limitation the rights to use, copy, modify, merge, publish,
;distribute, sublicense, and/or sell copies of the Software, and to
;permit persons to whom the Software is furnished to do so, subject to
;the following conditions:
;
;The above copyright notice and this permission notice shall be
;included in all copies or substantial portions of the Software.
;
;THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
;EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
;MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
;IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
;CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
;TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
;SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

(ns org.mathpiper.studentschedule.completescheduleengine.conflict_engine)

(use 'org.mathpiper.studentschedule.completescheduleengine.schedule_engine)

(defn conflict-count [course-number-1 course-number-2 course-map]
  
  (let
    [course-sections-1 (get-open-sections course-number-1 course-map)
     course-sections-2 (get-open-sections course-number-2 course-map)
     course-section-pairings (for [course-section-1 course-sections-1
                                   course-section-2 course-sections-2
                                   ] [course-section-1 course-section-2] )
     
     total-course-section-pairings (count course-section-pairings)
     illegal-course-section-pairings (count (filter #(overlap? (nth % 0) (nth % 1) course-map) course-section-pairings))
     legal-course-seciont-pairings (count (filter #(not (overlap? (nth % 0) (nth % 1) course-map)) course-section-pairings))
     
     
     ]
    
    [total-course-section-pairings
    illegal-course-section-pairings
    legal-course-seciont-pairings]
    
    )
  
  )

(defn conflict-count-vector
  
"   The function conflict-count-vector takes a list of course numbers in the form of keys
 , such as:

[:MATH1010 :ENGL1101 :MATH0099]

The function returns a list of lists of vectors in the form:

(([18 2 16] [198 23 175]) ([44 2 42]))

Where the first list in the output list from conflict-count-vector is the conflict vectors for the first course number in course-list
paired with the the secound, third and so on course numbers in course-list, and the secound list in the output list from
conflict-count-vector is the conflict vectors for the second course number in course-list paired with  the the third, fourth and so
on course numbers in course-list. In general, the Nth list in the retured list contains the conflict vector for the Nth course-number
in course-list paired with ever course number greater than Nth place in course-list.
     The conflict vector is given in the form:
 
[198 23 175]

Where the first place in the conflict vector is the number of possible section permutaions between the two course numbers for which
the vector summerizes. The second place is the number of illgal pairings, and the third plact is the number of legal section pairings.


"
  
  [course-list course-map]
  
  (let [course-pairings (for [index-1 (range 0 (- (count course-list) 1) )]
                          (for [index-2 (range (+ index-1 1)  (count course-list))]
                            (conflict-count (nth course-list index-1) (nth course-list index-2) course-map))
                          )
        
        ] course-pairings
    
    )
  
  
  
  
  )